<?php

namespace TDevAgency\Cms\Providers;

use DaveJamesMiller\Breadcrumbs\Facades\Breadcrumbs;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Blade;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\File;
use Illuminate\Support\ServiceProvider;
use TDevAgency\Cms\Commands\AddUser;
use TDevAgency\Cms\Commands\CmsInstall;
use TDevAgency\Cms\Commands\PublicAssets;
use TDevAgency\Cms\Models\BaseModels\AdminUser;
use TDevAgency\Cms\Observers\AdminUserObserver;

class CmsServiceProvider extends ServiceProvider
{

    /**
     * @var string
     */
    private $translationsPath;

    /**
     * var string
     */

    private $viewsPath;
    /**
     * @var string
     */
    private $migrationsPath;
    /**
     * @var string
     */
    private $noEditableTranslationsPath;

    public function __construct($app)
    {
        parent::__construct($app);

        $this->noEditableTranslationsPath = TDEV_AGENCY_CMS_ROOT . 'resources' . DIRECTORY_SEPARATOR . 'noneditable-lang';
        $this->translationsPath = TDEV_AGENCY_CMS_ROOT . 'resources' . DIRECTORY_SEPARATOR . 'lang';
        $this->viewsPath = TDEV_AGENCY_CMS_ROOT . 'resources' . DIRECTORY_SEPARATOR . 'views';
        $this->migrationsPath = TDEV_AGENCY_CMS_ROOT . 'database' . DIRECTORY_SEPARATOR . 'migrations';
    }

    /**
     * Register services.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfigFrom(
            TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'auth.php', 'auth'
        );
        $this->mergeConfigFrom(
            TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'scout.php', 'scout'
        );
        $this->mergeConfigFrom(
            TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'cms.php', 'cms'
        );
        $this->mergeConfigFrom(
            TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'cms_admin_sidebar.php', 'cms_admin_sidebar'
        );
        $this->mergeConfigFrom(
            TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'columnsortable.php', 'columnsortable'
        );
        $this->mergeConfigFrom(
            TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'breadcrumbs.php', 'breadcrumbs'
        );
    }

    /**
     * Merge the given configuration with the existing configuration.
     *
     * @param string $path
     * @param string $key
     * @return void
     */
    protected function mergeConfigFrom($path, $key): void
    {
        $config = $this->app['config']->get($key, []);
        $this->app['config']->set($key, $this->mergeConfigs($config, require $path));
    }

    /**
     * @param array $original
     * @param array $merging
     * @return array
     */
    protected function mergeConfigs(array $original, array $merging): array
    {
        $array = array_merge($original, $merging);
        foreach ($original as $key => $value) {
            if (!is_array($value)) {
                continue;
            }
            if (!Arr::exists($merging, $key)) {
                continue;
            }
            if (is_numeric($key)) {
                continue;
            }
            $array[$key] = $this->mergeConfigs($value, $merging[$key]);
        }
        return $array;
    }

    /**
     * Bootstrap services.
     *
     * @return void
     */
    public function boot()
    {

        AdminUser::observe(AdminUserObserver::class);

        $this->publishes([
            TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'cms.php' => config_path('cms.php'),
            TDEV_AGENCY_CMS_ROOT . 'config' . DIRECTORY_SEPARATOR . 'cms_admin_sidebar.php' => config_path('cms_admin_sidebar.php'),
        ], 'config');

        $this->publishes([
            $this->translationsPath => resource_path('lang/vendor/cms'),
        ], 'lang');

        $this->publishes([
            $this->viewsPath => resource_path('views/vendor/cms'),
        ], 'views');

        $this->loadMigrationsFrom($this->migrationsPath);
        $this->loadViewsFrom($this->viewsPath, 'cms');
        $this->loadTranslationsFrom($this->translationsPath, 'cms');
        $this->loadTranslationsFrom($this->noEditableTranslationsPath, 'cms-fixed');


        if (App::environment() !== 'production') {
            Blade::directive('xdebug', function ($expression) {
                return "<?php xdebug_break(); ?>";
            });
        }

        if ($this->app->runningInConsole()) {
            // Add commands here
            $this->commands([
                CmsInstall::class,
                AddUser::class,
                PublicAssets::class,
            ]);
        } else {


            if (class_exists('Breadcrumbs')) {
                require TDEV_AGENCY_CMS_ROOT . 'routes' . DIRECTORY_SEPARATOR . 'breadcrumbs.php';
                if (File::exists(base_path('routes/admin_breadcrumbs.php'))) {
                    require base_path('routes/admin_breadcrumbs.php');
                }
            }


            Breadcrumbs::macro('pageTitle', function () {
                $title = ($breadcrumb = Breadcrumbs::current()) ? (string)$breadcrumb->title : '';
                //TODO crash in edit page
                //if (($page = (int)request('page')) > 1) {
                //    $title .= ' - ' . trans('cms::breadcrumbs.page_no', ['page_no' => $page]);
                //}

                return $title;
            });

            Breadcrumbs::macro('pageMetaTitle', function () {
                $title = ($breadcrumb = Breadcrumbs::current()) ? (string)$breadcrumb->title . ' |' : '';
                if (($page = (int)request('page')) > 1) {
                    $title = sprintf('%s | %s', trans('cms::breadcrumbs.page_no', ['page_no' => $page]), $title);
                }

                return $title . ' | ' . Config::get('app.app_name');
            });
        }
    }

}
